"use strict";

exports.__esModule = true;
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MAIN_ENTRY = 'entry'; // This plugin creates a assets-manifest.json for all assets that are being output
// Because it`s hard to know whether a page has css before compilation, so we use this plugin to generate assets info during compilation

var RaxAssetsManifest =
/*#__PURE__*/
function () {
  function RaxAssetsManifest(options) {
    this.name = '[name]';
    this.options = options;
  }

  var _proto = RaxAssetsManifest.prototype;

  _proto.apply = function apply(compiler) {
    var _this = this;

    compiler.hooks.emit.tapAsync('RaxAssetsManifestPlugin', function (compilation, callback) {
      var chunks = compilation.chunks;
      var assetsMap = {};
      var mainChunk = chunks.find(function (c) {
        return c.name === MAIN_ENTRY;
      });
      var mainAssets = mainChunk ? mainChunk.files : [];
      chunks.map(function (chunk) {
        if (chunk.name === MAIN_ENTRY) {
          return;
        }

        var assets = mainAssets.concat(chunk.files);
        var publicPath = _this.options.publicPath || './';
        var styles = [];
        var scripts = [];
        assets.map(function (file) {
          if (/\.css$/.exec(file)) {
            styles.push(_path.default.join(publicPath, file));
            return;
          }

          if (/\.js$/.exec(file)) {
            scripts.push(_path.default.join(publicPath, file));
          }
        });
        assetsMap[chunk.name] = {
          styles: styles,
          scripts: scripts
        };
      });

      _fs.default.writeFileSync(_this.options.dist, JSON.stringify(assetsMap, null, 2));

      callback();
    });
  };

  return RaxAssetsManifest;
}();

exports.default = RaxAssetsManifest;