"use strict";

var fs = require('fs');

var path = require('path');

var webpack = require('webpack');

var _require = require('rax'),
    createElement = _require.createElement;

var renderer = require('rax-server-renderer');

var interopRequire = require('./interopRequire');

var getAssets = require('./getAssets');

var getWebpackNodeConfig = require('./getWebpackNodeConfig');

var DocumentHandler =
/*#__PURE__*/
function () {
  function DocumentHandler(options) {
    this.options = options;
    var _this$options = this.options,
        appDocument = _this$options.appDocument,
        appHtml = _this$options.appHtml,
        appDirectory = _this$options.appDirectory;
    this.documentJsFilePath;
    this.withDocumentJs = fs.existsSync(appDocument) || !fs.existsSync(appHtml); // Temp File

    this.tempHtmlFileName = '_document';
    this.tempHtmlFilePath = path.resolve(appDirectory, '.temp', this.tempHtmlFileName + '.js');

    if (this.withDocumentJs) {
      this.documentJsFilePath = fs.existsSync(appDocument) ? appDocument : require.resolve('rax-pwa/lib/Document');
    }
  }

  var _proto = DocumentHandler.prototype;

  _proto.build = function build(next) {
    var appDirectory = this.options.appDirectory;
    var webpackHtmlConfig = getWebpackNodeConfig(appDirectory);
    webpackHtmlConfig.entry[this.tempHtmlFileName] = this.documentJsFilePath;
    webpack(webpackHtmlConfig).run(function () {
      next('document');
    });
  };

  _proto.getDocument = function getDocument(compilation, appShellTemplate, skeletonTemplate, title) {
    if (appShellTemplate === void 0) {
      appShellTemplate = '';
    }

    if (skeletonTemplate === void 0) {
      skeletonTemplate = '';
    }

    if (title === void 0) {
      title = 'WebApp';
    }

    var _htmlValue;

    var _htmlPath;

    var _htmlAssets = getAssets(compilation);

    var appHtml = this.options.appHtml;

    if (!this.withDocumentJs) {
      // Use public/index.html
      _htmlPath = appHtml;
      _htmlValue = fs.readFileSync(_htmlPath, 'utf8');
      _htmlValue = _htmlValue.replace(/<div(.*?) id=\"root\">(.*?)<\/div>/, "<div id=\"root\">" + appShellTemplate + "</div>");

      if (skeletonTemplate) {
        _htmlValue = _htmlValue.replace('</body>', "<script>" + skeletonTemplate + "</script></body>");
      }

      var jsTagStr = _htmlAssets.js.map(function (src) {
        return "<script src=\"" + src + "\"></script>";
      }).join('') || '';
      var cssTagStr = _htmlAssets.css.map(function (src) {
        return "<link rel=\"stylesheet\" href=\"" + src + "\" />";
      }).join('') || '';
      _htmlValue = _htmlValue.replace('</head>', cssTagStr + "</head>").replace('</body>', jsTagStr + "</body>");
    } else {
      // Use document.js
      _htmlPath = this.documentJsFilePath;
      _htmlValue = renderer.renderToString(createElement(interopRequire(eval(fs.readFileSync(this.tempHtmlFilePath, 'utf-8'))), {
        scripts: _htmlAssets.js,
        styles: _htmlAssets.css,
        title: title,
        pageData: JSON.stringify({}),
        pageHtml: appShellTemplate
      }));

      if (skeletonTemplate) {
        _htmlValue = _htmlValue.replace('<script', "<script>" + skeletonTemplate + "</script><script");
      }
    }

    return {
      html: _htmlValue,
      path: _htmlPath
    };
  };

  return DocumentHandler;
}();

module.exports = DocumentHandler;