"use strict";

var getAssets = function getAssets(compilation) {
  var compilationHash = compilation.hash;
  var entryNames = Array.from(compilation.entrypoints.keys());
  var publicPath = compilation.mainTemplate.getPublicPath({
    hash: compilationHash
  });

  if (publicPath.length && publicPath.substr(-1, 1) !== '/') {
    publicPath += '/';
  }

  var assets = {
    publicPath: publicPath,
    js: [],
    css: []
  }; // Extract paths to .js, and .css files from the current compilation

  var entryPointPublicPathMap = {};
  var extensionRegexp = /\.(css|js)(\?|$)/;

  for (var i = 0; i < entryNames.length; i++) {
    var entryName = entryNames[i];
    var entryPointFiles = compilation.entrypoints.get(entryName).getFiles(); // Prepend the publicPath and append the hash depending on the

    var entryPointPublicPaths = entryPointFiles.map(function (chunkFile) {
      var entryPointPublicPath = publicPath + chunkFile;
      return entryPointPublicPath;
    });
    entryPointPublicPaths.forEach(function (entryPointPublicPath) {
      var extMatch = extensionRegexp.exec(entryPointPublicPath); // Skip if the public path is not a .css, .js file

      if (!extMatch) {
        return;
      } // Skip if this file is already known


      if (entryPointPublicPathMap[entryPointPublicPath]) {
        return;
      }

      entryPointPublicPathMap[entryPointPublicPath] = true;
      assets[extMatch[1]].push(entryPointPublicPath);
    });
  }

  return assets;
};

module.exports = getAssets;