"use strict";

var getSPAEntryCodeStr = function getSPAEntryCodeStr(options) {
  /**
   * Generate index.js for SPA
   * Features to be implemented:
   * 1. Insert App Shell and SPA router into the template
   * 2. receive data injected by SSR
   */
  var appConfig = options.appConfig,
      pathConfig = options.pathConfig,
      pagesConfig = options.pagesConfig,
      withSSR = options.withSSR,
      withAppShell = options.withAppShell;
  var importsCodeStr = '';
  var renderCodeStr = '';
  var pagesConfigCodeStr = ''; // process router config

  Object.keys(pagesConfig).forEach(function (pageName) {
    if (pageName === '_error') return;
    pagesConfigCodeStr += "\n      " + pageName + ": {\n        path: '" + pagesConfig[pageName].path + "', \n        regexp: " + pagesConfig[pageName]._regexp + ",\n        pageAlive: " + !!pagesConfig[pageName].pageAlive + ",\n        component: function() { return import(/* webpackChunkName: \"" + pageName + "\" */ '" + pathConfig.appSrc + "/pages/" + pageName + "/index') },\n        " + (pagesConfig[pageName].skeleton ? "skeleton: \"" + pagesConfig[pageName].skeleton + "\"," : '') + "\n        " + (pagesConfig[pageName].title || appConfig.title ? "title: \"" + (pagesConfig[pageName].title || appConfig.title) + "\"," : '') + "\n      },\n    ";
  });

  if (pagesConfig._error) {
    pagesConfigCodeStr += "\n      _error: {\n        component: function() {return import(/* webpackChunkName: \"_error\" */ '" + pathConfig.appSrc + "/pages/_error/index')}\n      }\n    ";
  } // process App Shell


  if (withAppShell) {
    importsCodeStr += "import Shell from '" + pathConfig.appShell + "';";
    renderCodeStr += 'render(<Shell Component={PageComponent} {...initialProps} />, document.getElementById("root"), { driver: DriverDOM, hydrate: true });';
  } else {
    renderCodeStr += 'render(<PageComponent {...initialProps} />, document.getElementById("root"), { driver: DriverDOM, hydrate: true });';
  }

  return "\n    import * as DriverDOM from 'driver-dom';\n    import { createElement, render, useState } from 'rax';\n    import { createRouter, getCurrentComponent } from 'rax-pwa';\n    " + importsCodeStr + "\n\n    let initialProps;\n    const pagesConfig = {\n      " + pagesConfigCodeStr + "\n    };\n    \n    const app = function() {\n      // Clear skeleton\n      if (document.getElementById('root-page')) {\n        document.getElementById('root-page').innerHTML = '';\n      }\n      \n      try {\n        initialProps = JSON.parse(document.querySelector(\"[data-from='server']\").innerHTML);\n      } catch (e) {\n        initialProps = {};\n      }\n\n      // In Code Splitting mode, the <Router /> is not rendering the routing content for the first time, result in unsuccessful hydrate components. \n      // Match the first component for hydrate\n      getCurrentComponent(pagesConfig, " + withSSR + ")().then(function(InitialComponent) {\n        if (InitialComponent === null) {\n          document.getElementById('root').innerHTML = '';\n          renderApp(InitialComponent);\n        } else {\n          InitialComponent = InitialComponent.__esModule ? InitialComponent.default : InitialComponent;\n          if(!" + withSSR + " && InitialComponent.getInitialProps) {\n            InitialComponent.getInitialProps().then(function(props) {\n              renderApp(InitialComponent, props);\n            }).catch(function(e){\n              renderApp(InitialComponent);  \n            });\n          } else {\n            renderApp(InitialComponent);\n          }\n        }\n      });\n    };\n\n    const renderApp = function(InitialComponent, initialComponentProps) {\n      const Router = createRouter(pagesConfig, " + withSSR + ", InitialComponent, initialComponentProps);\n      const PageComponent = function(props) {\n        " + (withAppShell && !withSSR ? 'return <div id="root-page" ><Router {...props}/></div>;' : 'return <Router {...props}/>') + "      \n      };\n      " + renderCodeStr + "\n    };\n \n    " + withSSR + "? window.addEventListener(\"load\", app) : app();\n  ";
};

module.exports = getSPAEntryCodeStr;