"use strict";

/**
 * Rax SPA Plugin
 * Compile the Router and generate the SPA entry file.
 *
 */
var fs = require('fs');

var path = require('path');

var qs = require('querystring');

var _require = require('../env'),
    isProductionMode = _require.isProductionMode;

var mkTempDir = require('../mkTempDir');

var getSPAPagesConfig = require('../getSPAPagesConfig');

var getSPAEntryCodeStr = require('../getSPAEntryCodeStr');

var RaxPWAPlugin =
/*#__PURE__*/
function () {
  function RaxPWAPlugin(options) {
    this.options = options;
  }

  var _proto = RaxPWAPlugin.prototype;

  _proto.apply = function apply(compiler) {
    var _this$options = this.options,
        appConfig = _this$options.appConfig,
        pathConfig = _this$options.pathConfig; // Mark optimization function points on PWA

    var withAppShell = fs.existsSync(pathConfig.appShell);
    var withSSR = !!appConfig.ssr;
    var withSPA = !!appConfig.spa;
    var withDocumentJs = fs.existsSync(pathConfig.appDocument) || !fs.existsSync(pathConfig.appHtml); // Temp file

    var tempIndexFileName = '_index';
    var tempIndexFilePath = path.resolve(pathConfig.appDirectory, '.temp', tempIndexFileName + '.js'); // Mark the current environment

    var isProductionLikeMode = isProductionMode(compiler); // Make temp directory

    mkTempDir(pathConfig.appDirectory);
    /**
     * Project Code pre-processing when SPA is turned on
     * 1. Update the project entry file from multiple to one main entry
     * 2. prepare the code for the main entry file
     * 3. prepare the skeleton map
     * 4. write temporary files, entry.
     */

    if (withSPA) {
      var pagesConfig = getSPAPagesConfig(appConfig, pathConfig);
      var newEntry = {
        entry: tempIndexFilePath
      }; // Dev mode for hot reload

      if (!isProductionLikeMode && withDocumentJs && fs.existsSync(pathConfig.appDocument)) {
        newEntry._document = pathConfig.appDocument;
      }

      var entryCodeStr = getSPAEntryCodeStr({
        appConfig: appConfig,
        pathConfig: pathConfig,
        pagesConfig: pagesConfig,
        withSSR: withSSR,
        withAppShell: withAppShell
      });
      fs.writeFileSync(tempIndexFilePath, entryCodeStr);
      compiler.options.entry = newEntry;
    } else if (withSSR) {
      // SSR entry add ClientLoader
      var ClientLoader = require.resolve('../loaders/ClientLoader.js');

      var entries = compiler.options.entry;
      Object.keys(entries).forEach(function (key) {
        var mainEntryFile = entries[key][0];

        if (mainEntryFile.indexOf(ClientLoader) === -1) {
          entries[key][0] = ClientLoader + "?" + qs.stringify({
            ssr: true,
            withAppShell: withAppShell
          }) + "!" + mainEntryFile;
        }
      });
    }
  };

  return RaxPWAPlugin;
}();

module.exports = RaxPWAPlugin;